@@ -14,7 +14,9 @@ from util import dbg, err
 import config
 from keybindings import Keybindings, KeymapError
 from translation import _
+from encoding import TerminatorEncoding
 from terminator import Terminator
+from plugin import PluginRegistry
 
 def color2hex(widget):
     """Pull the colour values out of a Gtk ColorPicker widget and return them
@@ -26,6 +28,8 @@ def color2hex(widget):
 class PrefsEditor:
     """Class implementing the various parts of the preferences editor"""
     config = None
+    registry = None
+    plugins = None
     keybindings = None
     window = None
     builder = None
@@ -118,6 +122,9 @@ class PrefsEditor:
                         'ungroup_tab'      : 'Ungroup terminals in tab',
                         'new_window'       : 'Create a new window',
                         'new_terminator'   : 'Spawn a new Terminator process',
+                        'broadcast_off'    : 'Don\'t broadcast key presses',
+                        'broadcast_group'  : 'Broadcast key presses to group',
+                        'broadcast_all'    : 'Broadcast key events to all'
             }
 
     def __init__ (self, term):
@@ -147,8 +154,7 @@ class PrefsEditor:
             self.set_values()
         except Exception, e:
             err('Unable to set values: %s' % e)
-        finally:
-            self.config.uninhibit_save()
+        self.config.uninhibit_save()
 
     def on_closebutton_clicked(self, _button):
         """Close the window"""
@@ -204,9 +210,14 @@ class PrefsEditor:
             active = 2
         elif option == 'right':
             active = 3
+        elif option == 'hidden':
+            active = 4
         else:
             active = 0
         widget.set_active(active)
+        # DBus Server
+        widget = guiget('dbuscheck')
+        widget.set_active(self.config['dbus'])
 
         ## Profile tab
         # Populate the profile list
@@ -261,7 +272,23 @@ class PrefsEditor:
                              keyval, mask])
 
         ## Plugins tab
-        # FIXME: Implement this
+        # Populate the plugin list
+        widget = guiget('pluginlist')
+        liststore = widget.get_model()
+        self.registry = PluginRegistry()
+        self.pluginiters = {}
+        pluginlist = self.registry.get_available_plugins()
+        self.plugins = {}
+        for plugin in pluginlist:
+            self.plugins[plugin] = self.registry.is_enabled(plugin)
+
+        for plugin in self.plugins:
+            self.pluginiters[plugin] = liststore.append([plugin,
+                                             self.plugins[plugin]])
+        selection = widget.get_selection()
+        selection.connect('changed', self.on_plugin_selection_changed)
+        if len(self.pluginiters) > 0:
+            selection.select_iter(liststore.get_iter_first())
 
     def set_profile_values(self, profile):
         """Update the profile values for a given profile"""
@@ -287,6 +314,9 @@ class PrefsEditor:
         # Allow bold text
         widget = guiget('allow_bold_checkbutton')
         widget.set_active(self.config['allow_bold'])
+        # Anti-alias
+        widget = guiget('antialias_checkbutton')
+        widget.set_active(self.config['antialias'])
         # Icon terminal bell
         widget = guiget('icon_bell_checkbutton')
         widget.set_active(self.config['icon_bell'])
@@ -302,6 +332,9 @@ class PrefsEditor:
         # Show titlebar
         widget = guiget('show_titlebar')
         widget.set_active(self.config['show_titlebar'])
+        # Copy on selection
+        widget = guiget('copy_on_selection')
+        widget.set_active(self.config['copy_on_selection'])
         # Word chars
         widget = guiget('word_chars_entry')
         widget.set_text(self.config['word_chars'])
@@ -481,6 +514,23 @@ class PrefsEditor:
             widget.set_active(3)
         else:
             widget.set_active(0)
+        # Encoding
+        rowiter = None
+        widget = guiget('encoding_combobox')
+        encodingstore = guiget('EncodingListStore')
+        value = self.config['encoding']
+        encodings = TerminatorEncoding().get_list()
+        encodings.sort(lambda x, y: cmp(x[2].lower(), y[2].lower()))
+
+        for encoding in encodings:
+            if encoding[1] is None:
+                continue
+            
+            label = "%s %s" % (encoding[2], encoding[1])
+            rowiter = encodingstore.append([label, encoding[1]])
+
+            if encoding[1] == value:
+                widget.set_active_iter(rowiter)
 
     def set_layout(self, layout_name):
         """Set a layout"""
@@ -491,6 +541,11 @@ class PrefsEditor:
         self.config['geometry_hinting'] = widget.get_active()
         self.config.save()
 
+    def on_dbuscheck_toggled(self, widget):
+        """DBus server setting changed"""
+        self.config['dbus'] = widget.get_active()
+        self.config.save()
+
     def on_winbordercheck_toggled(self, widget):
         """Window border setting changed"""
         self.config['borderless'] = not widget.get_active()
@@ -501,11 +556,21 @@ class PrefsEditor:
         self.config['allow_bold'] = widget.get_active()
         self.config.save()
 
+    def on_antialias_checkbutton_toggled(self, widget):
+        """Anti-alias setting changed"""
+        self.config['antialias'] = widget.get_active()
+        self.config.save()
+
     def on_show_titlebar_toggled(self, widget):
         """Show titlebar setting changed"""
         self.config['show_titlebar'] = widget.get_active()
         self.config.save()
 
+    def on_copy_on_selection_toggled(self, widget):
+        """Copy on selection setting changed"""
+        self.config['copy_on_selection'] = widget.get_active()
+        self.config.save()
+
     def on_cursor_blink_toggled(self, widget):
         """Cursor blink setting changed"""
         self.config['cursor_blink'] = widget.get_active()
@@ -589,6 +654,15 @@ class PrefsEditor:
         self.config['backspace_binding'] = value
         self.config.save()
 
+    def on_encoding_combobox_changed(self, widget):
+        """Encoding setting changed"""
+        selected = widget.get_active_iter()
+        liststore = widget.get_model()
+        value = liststore.get_value(selected, 1)
+
+        self.config['encoding'] = value
+        self.config.save()
+
     def on_scrollback_lines_spinbutton_value_changed(self, widget):
         """Scrollback lines setting changed"""
         value = widget.get_value_as_int()
@@ -796,6 +870,8 @@ class PrefsEditor:
             value = 'left'
         elif selected == 3:
             value = 'right'
+        elif selected == 4:
+            value = 'hidden'
         else:
             value = 'top'
         self.config['tab_position'] = value
@@ -986,6 +1062,48 @@ class PrefsEditor:
         else:
             widget.set_sensitive(True)
 
+    def on_plugin_selection_changed(self, selection):
+        """A different plugin was selected"""
+        (listmodel, rowiter) = selection.get_selected()
+        if not rowiter:
+            # Something is wrong, just jump to the first item in the list
+            treeview = selection.get_tree_view()
+            liststore = treeview.get_model()
+            selection.select_iter(liststore.get_iter_first())
+            return
+        plugin = listmodel.get_value(rowiter, 0)
+        self.set_plugin(plugin)
+        self.previous_plugin_selection = plugin
+
+        widget = self.builder.get_object('plugintogglebutton')
+
+    def on_plugin_toggled(self, cell, path):
+        """A plugin has been enabled or disabled"""
+        treeview = self.builder.get_object('pluginlist')
+        model = treeview.get_model()
+        plugin = model[path][0]
+
+        if not self.plugins[plugin]:
+            # Plugin is currently disabled, load it
+            self.registry.enable(plugin)
+        else:
+            # Plugin is currently enabled, unload it
+            self.registry.disable(plugin)
+
+        self.plugins[plugin] = not self.plugins[plugin]
+        # Update the treeview
+        model[path][1] = self.plugins[plugin]
+
+        enabled_plugins = [x for x in self.plugins if self.plugins[x] == True]
+        self.config['enabled_plugins'] = enabled_plugins
+        self.config.save()
+
+    def set_plugin(self, plugin):
+        """Show the preferences for the selected plugin, if any"""
+        pluginpanelabel = self.builder.get_object('pluginpanelabel')
+        pluginconfig = self.config.plugin_get_config(plugin)
+        # FIXME: Implement this, we need to auto-construct a UI for the plugin
+
     def on_profile_name_edited(self, cell, path, newtext):
         """Update a profile name"""
         oldname = cell.get_property('text')
