@@ -105,6 +105,7 @@ class Terminal(gtk.VBox):
         self.terminator = Terminator()
         self.terminator.register_terminal(self)
 
+        # FIXME: Surely these should happen in Terminator::register_terminal()?
         self.connect('enumerate', self.terminator.do_enumerate)
         self.connect('focus-in', self.terminator.focus_changed)
 
@@ -151,6 +152,7 @@ class Terminal(gtk.VBox):
 
         self.connect_signals()
 
+        os.putenv('TERM', 'xterm')
         os.putenv('COLORTERM', 'gnome-terminal')
 
         env_proxy = os.getenv('http_proxy')
@@ -158,10 +160,18 @@ class Terminal(gtk.VBox):
             if self.config['http_proxy'] and self.config['http_proxy'] != '':
                 os.putenv('http_proxy', self.config['http_proxy'])
 
-    def set_profile(self, _widget, profile):
+    def get_vte(self):
+        """This simply returns the vte widget we are using"""
+        return(self.vte)
+
+    def force_set_profile(self, widget, profile):
+        """Forcibly set our profile"""
+        self.set_profile(widget, profile, True)
+
+    def set_profile(self, _widget, profile, force=False):
         """Set our profile"""
         if profile != self.config.get_profile():
-            self.config.set_profile(profile)
+            self.config.set_profile(profile, force)
             self.reconfigure()
 
     def get_profile(self):
@@ -259,12 +269,31 @@ class Terminal(gtk.VBox):
                 for urlplugin in plugins:
                     name = urlplugin.handler_name
                     match = urlplugin.match
+                    if name in self.matches:
+                        dbg('Terminal::update_matches: refusing to add \
+duplicate match %s' % name)
+                        continue
                     self.matches[name] = self.vte.match_add(match)
                     dbg('Terminal::update_matches: added plugin URL handler \
 for %s (%s)' % (name, urlplugin.__class__.__name__))
             except Exception, ex:
                 err('Terminal::update_url_matches: %s' % ex)
-            
+
+    def match_add(self, name, match):
+        """Register a URL match"""
+        if name in self.matches:
+            err('Terminal::match_add: Refusing to create duplicate match %s' % name)
+            return
+        self.matches[name] = self.vte.match_add(match)
+
+    def match_remove(self, name):
+        """Remove a previously registered URL match"""
+        if name not in self.matches:
+            err('Terminal::match_remove: Unable to remove non-existent match %s' % name)
+            return
+        self.vte.match_remove(self.matches[name])
+        del(self.matches[name])
+
     def connect_signals(self):
         """Connect all the gtk signals and drag-n-drop mechanics"""
 
@@ -557,7 +586,7 @@ for %s (%s)' % (name, urlplugin.__class__.__name__))
                     font = self.config.get_system_font()
                 else:
                     font = self.config['font']
-                self.vte.set_font(pango.FontDescription(font))
+                self.set_font(pango.FontDescription(font))
             except:
                 pass
         self.vte.set_allow_bold(self.config['allow_bold'])
@@ -954,6 +983,10 @@ for %s (%s)' % (name, urlplugin.__class__.__name__))
         """Inform other parts of the application when focus is received"""
         self.emit('focus-in')
 
+    def on_window_focus_out(self):
+        """Update our UI when the window loses focus"""
+        self.titlebar.update('window-focus-out')
+
     def scrollbar_jump(self, position):
         """Move the scrollbar to a particular row"""
         self.scrollbar.set_value(position)
@@ -1039,7 +1072,7 @@ for %s (%s)' % (name, urlplugin.__class__.__name__))
             return
         new_font.set_size(new_size)
         dbg('setting new font: %s' % new_font)
-        self.vte.set_font(new_font)
+        self.set_font(new_font)
 
     def is_zoomed(self):
         """Determine if we are a zoomed terminal"""
@@ -1192,8 +1225,12 @@ for %s (%s)' % (name, urlplugin.__class__.__name__))
             oldstyle = True
 
         if oldstyle == False:
-            gtk.show_uri(None, url, gtk.gdk.CURRENT_TIME)
-        else:
+            try:
+                gtk.show_uri(None, url, gtk.gdk.CURRENT_TIME)
+            except:
+                oldstyle = True
+
+        if oldstyle == True:
             dbg('Old gtk (%s,%s,%s), calling xdg-open' % gtk.gtk_version)
             try:
                 subprocess.Popen(["xdg-open", url])
@@ -1234,7 +1271,7 @@ for %s (%s)' % (name, urlplugin.__class__.__name__))
             fontsize += pango.SCALE
 
         pangodesc.set_size(fontsize)
-        self.vte.set_font(pangodesc)
+        self.set_font(pangodesc)
         self.custom_font_size = fontsize
 
     def zoom_orig(self):
@@ -1244,11 +1281,21 @@ for %s (%s)' % (name, urlplugin.__class__.__name__))
         else:
             font = self.config['font']
         dbg("Terminal::zoom_orig: restoring font to: %s" % font)
-        self.vte.set_font(pango.FontDescription(font))
+        self.set_font(pango.FontDescription(font))
         self.custom_font_size = None
 
+    def set_font(self, fontdesc):
+        """Set the font we want in VTE"""
+        antialias = self.config['antialias']
+        if antialias:
+            antialias = vte.ANTI_ALIAS_FORCE_ENABLE
+        else:
+            antialias = vte.ANTI_ALIAS_FORCE_DISABLE
+        self.vte.set_font_full(fontdesc, antialias)
+
     def get_cursor_position(self):
         """Return the co-ordinates of our cursor"""
+        # FIXME: THIS METHOD IS DEPRECATED AND UNUSED
         col, row = self.vte.get_cursor_position()
         width = self.vte.get_char_width()
         height = self.vte.get_char_height()
@@ -1276,10 +1323,15 @@ for %s (%s)' % (name, urlplugin.__class__.__name__))
         layout['type'] = 'Terminal'
         layout['parent'] = parent
         layout['order'] = child_order
+        if self.group:
+            layout['group'] = self.group
         profile = self.get_profile()
         if layout != "default":
             # There's no point explicitly noting default profiles
             layout['profile'] = profile
+        title = self.titlebar.get_custom_string()
+        if title:
+            layout['title'] = title
         name = 'terminal%d' % count
         count = count + 1
         global_layout[name] = layout
@@ -1287,8 +1339,18 @@ for %s (%s)' % (name, urlplugin.__class__.__name__))
 
     def create_layout(self, layout):
         """Apply our layout"""
+        dbg('Setting layout')
         if layout.has_key('command') and layout['command'] != '':
             self.layout_command = layout['command']
+        if layout.has_key('profile') and layout['profile'] != '':
+            if layout['profile'] in self.config.list_profiles():
+                self.set_profile(self, layout['profile'])
+        if layout.has_key('group') and layout['group'] != '':
+            # This doesn't need/use self.titlebar, but it's safer than sending
+            # None
+            self.really_create_group(self.titlebar, layout['group'])
+        if layout.has_key('title') and layout['title'] != '':
+            self.titlebar.set_custom_string(layout['title'])
 
     # There now begins a great list of keyboard event handlers
     def key_zoom_in(self):
@@ -1301,7 +1363,7 @@ for %s (%s)' % (name, urlplugin.__class__.__name__))
         self.vte.copy_clipboard()
 
     def key_paste(self):
-        self.vte.paste_clipboard()
+        self.paste_clipboard()
 
     def key_toggle_scrollbar(self):
         self.do_scrollbar_toggle()
@@ -1434,7 +1496,7 @@ for %s (%s)' % (name, urlplugin.__class__.__name__))
         self.emit('ungroup-tab')
 
     def key_new_window(self):
-        self.terminator.new_window()
+        self.terminator.new_window(self.terminator.pid_cwd(self.pid))
 
     def key_new_terminator(self):
         cmd = sys.argv[0]
@@ -1448,7 +1510,17 @@ for %s (%s)' % (name, urlplugin.__class__.__name__))
                 return False
           
         dbg("Terminal::key_new_window: Spawning: %s" % cmd)
-        subprocess.Popen([cmd, ])
+        subprocess.Popen([cmd, '-u'])
+
+    def key_broadcast_off(self):
+        self.set_groupsend(None, self.terminator.groupsend_type['off'])
+
+    def key_broadcast_group(self):
+        self.set_groupsend(None, self.terminator.groupsend_type['group'])
+
+    def key_broadcast_all(self):
+        self.set_groupsend(None, self.terminator.groupsend_type['all'])
+
 # End key events
 
 gobject.type_register(Terminal)
